@@ -1,49 +1,58 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 """
-    Tests for cssselect
-    ===================
+Tests for cssselect
+===================
 
-    These tests can be run either by py.test or by the standard library's
-    unittest. They use plain ``assert`` statements and do little reporting
-    themselves in case of failure.
+These tests can be run either by py.test or by the standard library's
+unittest. They use plain ``assert`` statements and do little reporting
+themselves in case of failure.
 
-    Use py.test to get fancy error reporting and assert introspection.
+Use py.test to get fancy error reporting and assert introspection.
 
 
-    :copyright: (c) 2007-2012 Ian Bicking and contributors.
-                See AUTHORS for more details.
-    :license: BSD, see LICENSE for more details.
+:copyright: (c) 2007-2012 Ian Bicking and contributors.
+See AUTHORS for more details.
+:license: BSD, see LICENSE for more details.
 
 """
 
+from __future__ import annotations
+
 import sys
 import typing
 import unittest
-from typing import List, Optional, Sequence, Tuple
+from typing import TYPE_CHECKING
 
+import pytest
 from lxml import etree, html
+
 from cssselect import (
-    parse,
+    ExpressionError,
     GenericTranslator,
     HTMLTranslator,
     SelectorSyntaxError,
-    ExpressionError,
+    parse,
 )
 from cssselect.parser import (
-    tokenize,
-    parse_series,
-    PseudoElement,
-    FunctionalPseudoElement,
     Function,
+    FunctionalPseudoElement,
+    PseudoElement,
     Token,
+    parse_series,
+    tokenize,
 )
 from cssselect.xpath import XPathExpr
 
+if TYPE_CHECKING:
+    from collections.abc import Sequence
+
 
 class TestCssselect(unittest.TestCase):
     def test_tokenizer(self) -> None:
-        tokens = [str(item) for item in tokenize(r'E\ é > f [a~="y\"x"]:nth(/* fu /]* */-3.7)')]
+        tokens = [
+            str(item)
+            for item in tokenize(r'E\ é > f [a~="y\"x"]:nth(/* fu /]* */-3.7)')
+        ]
         assert tokens == [
             "<IDENT 'E é' at 0>",
             "<S ' ' at 4>",
@@ -66,13 +75,13 @@ class TestCssselect(unittest.TestCase):
         ]
 
     def test_parser(self) -> None:
-        def repr_parse(css: str) -> List[str]:
+        def repr_parse(css: str) -> list[str]:
             selectors = parse(css)
             for selector in selectors:
                 assert selector.pseudo_element is None
-            return [repr(selector.parsed_tree).replace("(u'", "('") for selector in selectors]
+            return [repr(selector.parsed_tree) for selector in selectors]
 
-        def parse_many(first: str, *others: str) -> List[str]:
+        def parse_many(first: str, *others: str) -> list[str]:
             result = repr_parse(first)
             for other in others:
                 assert repr_parse(other) == result
@@ -95,7 +104,9 @@ class TestCssselect(unittest.TestCase):
             "div\r>\n\n\n.foo",
             "div\f>\f.foo",
         ) == ["CombinedSelector[Element[div] > Class[Element[*].foo]]"]
-        assert parse_many("td.foo,.bar", "td.foo, .bar", "td.foo\t\r\n\f ,\t\r\n\f .bar") == [
+        assert parse_many(
+            "td.foo,.bar", "td.foo, .bar", "td.foo\t\r\n\f ,\t\r\n\f .bar"
+        ) == [
             "Class[Element[td].foo]",
             "Class[Element[*].bar]",
         ]
@@ -123,11 +134,15 @@ class TestCssselect(unittest.TestCase):
         assert parse_many("a[hreflang |= 'en']", "a[hreflang|=en]") == [
             "Attrib[Element[a][hreflang |= 'en']]"
         ]
-        assert parse_many("div:nth-child(10)") == ["Function[Element[div]:nth-child(['10'])]"]
+        assert parse_many("div:nth-child(10)") == [
+            "Function[Element[div]:nth-child(['10'])]"
+        ]
         assert parse_many(":nth-child(2n+2)") == [
             "Function[Element[*]:nth-child(['2', 'n', '+2'])]"
         ]
-        assert parse_many("div:nth-of-type(10)") == ["Function[Element[div]:nth-of-type(['10'])]"]
+        assert parse_many("div:nth-of-type(10)") == [
+            "Function[Element[div]:nth-of-type(['10'])]"
+        ]
         assert parse_many("div div:nth-of-type(10) .aclass") == [
             "CombinedSelector[CombinedSelector[Element[div] <followed> "
             "Function[Element[div]:nth-of-type(['10'])]] "
@@ -135,7 +150,9 @@ class TestCssselect(unittest.TestCase):
         ]
         assert parse_many("label:only") == ["Pseudo[Element[label]:only]"]
         assert parse_many("a:lang(fr)") == ["Function[Element[a]:lang(['fr'])]"]
-        assert parse_many('div:contains("foo")') == ["Function[Element[div]:contains(['foo'])]"]
+        assert parse_many('div:contains("foo")') == [
+            "Function[Element[div]:contains(['foo'])]"
+        ]
         assert parse_many("div#foobar") == ["Hash[Element[div]#foobar]"]
         assert parse_many("div:not(div.foo)") == [
             "Negation[Element[div]:not(Class[Element[div].foo])]"
@@ -170,18 +187,18 @@ class TestCssselect(unittest.TestCase):
         ]
 
     def test_pseudo_elements(self) -> None:
-        def parse_pseudo(css: str) -> List[Tuple[str, Optional[str]]]:
-            result: List[Tuple[str, Optional[str]]] = []
+        def parse_pseudo(css: str) -> list[tuple[str, str | None]]:
+            result: list[tuple[str, str | None]] = []
             for selector in parse(css):
                 pseudo = selector.pseudo_element
                 pseudo = str(pseudo) if pseudo else pseudo
                 # No Symbol here
-                assert pseudo is None or type(pseudo) is str
-                selector_as_str = repr(selector.parsed_tree).replace("(u'", "('")
+                assert pseudo is None or isinstance(pseudo, str)
+                selector_as_str = repr(selector.parsed_tree)
                 result.append((selector_as_str, pseudo))
             return result
 
-        def parse_one(css: str) -> Tuple[str, Optional[str]]:
+        def parse_one(css: str) -> tuple[str, str | None]:
             result = parse_pseudo(css)
             assert len(result) == 1
             return result[0]
@@ -242,7 +259,7 @@ class TestCssselect(unittest.TestCase):
 
         # Special cases for CSS 2.1 pseudo-elements are ignored by default
         for pseudo in ("after", "before", "first-line", "first-letter"):
-            (selector,) = parse("e:%s" % pseudo)
+            (selector,) = parse(f"e:{pseudo}")
             assert selector.pseudo_element == pseudo
             assert GenericTranslator().selector_to_xpath(selector, prefix="") == "e"
 
@@ -252,9 +269,8 @@ class TestCssselect(unittest.TestCase):
         (selector,) = parse("e::foo")
         assert selector.pseudo_element == "foo"
         assert tr.selector_to_xpath(selector, prefix="") == "e"
-        self.assertRaises(
-            ExpressionError, tr.selector_to_xpath, selector, translate_pseudo_elements=True
-        )
+        with pytest.raises(ExpressionError):
+            tr.selector_to_xpath(selector, translate_pseudo_elements=True)
 
         # Special test for the unicode symbols and ':scope' element if check
         # Errors if use repr() instead of __repr__()
@@ -262,7 +278,7 @@ class TestCssselect(unittest.TestCase):
         assert test_pseudo_repr(":scope") == "Pseudo[Element[*]:scope]"
 
     def test_specificity(self) -> None:
-        def specificity(css: str) -> Tuple[int, int, int]:
+        def specificity(css: str) -> tuple[int, int, int]:
             selectors = parse(css)
             assert len(selectors) == 1
             return selectors[0].specificity()
@@ -301,10 +317,14 @@ class TestCssselect(unittest.TestCase):
         assert specificity("foo::before") == (0, 0, 2)
         assert specificity("foo:empty::before") == (0, 1, 2)
 
-        assert specificity("#lorem + foo#ipsum:first-child > bar:first-line") == (2, 1, 3)
+        assert specificity("#lorem + foo#ipsum:first-child > bar:first-line") == (
+            2,
+            1,
+            3,
+        )
 
     def test_css_export(self) -> None:
-        def css2css(css: str, res: Optional[str] = None) -> None:
+        def css2css(css: str, res: str | None = None) -> None:
             selectors = parse(css)
             assert len(selectors) == 1
             assert selectors[0].canonical() == (res or css)
@@ -343,18 +363,19 @@ class TestCssselect(unittest.TestCase):
         css2css("foo > *")
 
     def test_parse_errors(self) -> None:
-        def get_error(css: str) -> Optional[str]:
+        def get_error(css: str) -> str | None:
             try:
                 parse(css)
             except SelectorSyntaxError:
-                # Py2, Py3, ...
-                return str(sys.exc_info()[1]).replace("(u'", "('")
+                return str(sys.exc_info()[1])
             return None
 
         assert get_error("attributes(href)/html/body/a") == (
             "Expected selector, got <DELIM '(' at 10>"
         )
-        assert get_error("attributes(href)") == ("Expected selector, got <DELIM '(' at 10>")
+        assert get_error("attributes(href)") == (
+            "Expected selector, got <DELIM '(' at 10>"
+        )
         assert get_error("html/body/a") == ("Expected selector, got <DELIM '/' at 4>")
         assert get_error(" ") == ("Expected selector, got <EOF at 1>")
         assert get_error("div, ") == ("Expected selector, got <EOF at 5>")
@@ -369,10 +390,14 @@ class TestCssselect(unittest.TestCase):
         assert get_error("[*]") == ("Expected '|', got <DELIM ']' at 2>")
         assert get_error("[foo|]") == ("Expected ident, got <DELIM ']' at 5>")
         assert get_error("[#]") == ("Expected ident or '*', got <DELIM '#' at 1>")
-        assert get_error("[foo=#]") == ("Expected string or ident, got <DELIM '#' at 5>")
+        assert get_error("[foo=#]") == (
+            "Expected string or ident, got <DELIM '#' at 5>"
+        )
         assert get_error("[href]a") == ("Expected selector, got <IDENT 'a' at 6>")
         assert get_error("[rel=stylesheet]") is None
-        assert get_error("[rel:stylesheet]") == ("Operator expected, got <DELIM ':' at 4>")
+        assert get_error("[rel:stylesheet]") == (
+            "Operator expected, got <DELIM ':' at 4>"
+        )
         assert get_error("[rel=stylesheet") == ("Expected ']', got <EOF at 15>")
         assert get_error(":lang(fr)") is None
         assert get_error(":lang(fr") == ("Expected an argument, got <EOF at 8>")
@@ -386,12 +411,20 @@ class TestCssselect(unittest.TestCase):
         assert get_error("li:before a") == (
             "Got pseudo-element ::before not at the end of a selector"
         )
-        assert get_error(":not(:before)") == ("Got pseudo-element ::before inside :not() at 12")
+        assert get_error(":not(:before)") == (
+            "Got pseudo-element ::before inside :not() at 12"
+        )
         assert get_error(":not(:not(a))") == ("Got nested :not()")
-        assert get_error(":is(:before)") == ("Got pseudo-element ::before inside function")
+        assert get_error(":is(:before)") == (
+            "Got pseudo-element ::before inside function"
+        )
         assert get_error(":is(a b)") == ("Expected an argument, got <IDENT 'b' at 6>")
-        assert get_error(":where(:before)") == ("Got pseudo-element ::before inside function")
-        assert get_error(":where(a b)") == ("Expected an argument, got <IDENT 'b' at 9>")
+        assert get_error(":where(:before)") == (
+            "Got pseudo-element ::before inside function"
+        )
+        assert get_error(":where(a b)") == (
+            "Expected an argument, got <IDENT 'b' at 9>"
+        )
         assert get_error(":scope > div :scope header") == (
             'Got immediate child pseudo-element ":scope" not at the start of a selector'
         )
@@ -416,7 +449,7 @@ class TestCssselect(unittest.TestCase):
         assert xpath("e[foo|bar]") == "e[@foo:bar]"
         assert xpath('e[foo="bar"]') == "e[@foo = 'bar']"
         assert xpath('e[foo~="bar"]') == (
-            "e[@foo and contains(" "concat(' ', normalize-space(@foo), ' '), ' bar ')]"
+            "e[@foo and contains(concat(' ', normalize-space(@foo), ' '), ' bar ')]"
         )
         assert xpath('e[foo^="bar"]') == ("e[@foo and starts-with(@foo, 'bar')]")
         assert xpath('e[foo$="bar"]') == (
@@ -424,7 +457,7 @@ class TestCssselect(unittest.TestCase):
         )
         assert xpath('e[foo*="bar"]') == ("e[@foo and contains(@foo, 'bar')]")
         assert xpath('e[hreflang|="en"]') == (
-            "e[@hreflang and (" "@hreflang = 'en' or starts-with(@hreflang, 'en-'))]"
+            "e[@hreflang and (@hreflang = 'en' or starts-with(@hreflang, 'en-'))]"
         )
 
         # --- nth-* and nth-last-* -------------------------------------
@@ -446,19 +479,29 @@ class TestCssselect(unittest.TestCase):
             "e[(count(preceding-sibling::*) >= 1) and "
             "((count(preceding-sibling::*) +2) mod 3 = 0)]"
         )
-        assert xpath("e:nth-child(3n-2)") == ("e[count(preceding-sibling::*) mod 3 = 0]")
+        assert xpath("e:nth-child(3n-2)") == (
+            "e[count(preceding-sibling::*) mod 3 = 0]"
+        )
         assert xpath("e:nth-child(-n+6)") == ("e[count(preceding-sibling::*) <= 5]")
 
         assert xpath("e:nth-last-child(1)") == ("e[count(following-sibling::*) = 0]")
-        assert xpath("e:nth-last-child(2n)") == ("e[(count(following-sibling::*) +1) mod 2 = 0]")
-        assert xpath("e:nth-last-child(2n+1)") == ("e[count(following-sibling::*) mod 2 = 0]")
+        assert xpath("e:nth-last-child(2n)") == (
+            "e[(count(following-sibling::*) +1) mod 2 = 0]"
+        )
+        assert xpath("e:nth-last-child(2n+1)") == (
+            "e[count(following-sibling::*) mod 2 = 0]"
+        )
         assert xpath("e:nth-last-child(2n+2)") == (
             "e[(count(following-sibling::*) >= 1) and "
             "((count(following-sibling::*) +1) mod 2 = 0)]"
         )
-        assert xpath("e:nth-last-child(3n+1)") == ("e[count(following-sibling::*) mod 3 = 0]")
+        assert xpath("e:nth-last-child(3n+1)") == (
+            "e[count(following-sibling::*) mod 3 = 0]"
+        )
         # represents the two last e elements
-        assert xpath("e:nth-last-child(-n+2)") == ("e[count(following-sibling::*) <= 1]")
+        assert xpath("e:nth-last-child(-n+2)") == (
+            "e[count(following-sibling::*) <= 1]"
+        )
 
         assert xpath("e:nth-of-type(1)") == ("e[count(preceding-sibling::e) = 0]")
         assert xpath("e:nth-last-of-type(1)") == ("e[count(following-sibling::e) = 0]")
@@ -486,24 +529,32 @@ class TestCssselect(unittest.TestCase):
         assert xpath("e:has(f)") == "e[descendant::f]"
         assert xpath("e:has(~ f)") == "e[following-sibling::f]"
         assert (
-            xpath("e:has(+ f)") == "e[following-sibling::*[(name() = 'f') and (position() = 1)]]"
+            xpath("e:has(+ f)")
+            == "e[following-sibling::*[(name() = 'f') and (position() = 1)]]"
         )
         assert xpath('e:contains("foo")') == ("e[contains(., 'foo')]")
         assert xpath("e:ConTains(foo)") == ("e[contains(., 'foo')]")
         assert xpath("e.warning") == (
-            "e[@class and contains(" "concat(' ', normalize-space(@class), ' '), ' warning ')]"
+            "e[@class and contains("
+            "concat(' ', normalize-space(@class), ' '), ' warning ')]"
         )
         assert xpath("e#myid") == ("e[@id = 'myid']")
-        assert xpath("e:not(:nth-child(odd))") == ("e[not(count(preceding-sibling::*) mod 2 = 0)]")
+        assert xpath("e:not(:nth-child(odd))") == (
+            "e[not(count(preceding-sibling::*) mod 2 = 0)]"
+        )
         assert xpath("e:nOT(*)") == ("e[0]")  # never matches
         assert xpath("e f") == ("e/descendant-or-self::*/f")
         assert xpath("e > f") == ("e/f")
-        assert xpath("e + f") == ("e/following-sibling::*[(name() = 'f') and (position() = 1)]")
+        assert xpath("e + f") == (
+            "e/following-sibling::*[(name() = 'f') and (position() = 1)]"
+        )
         assert xpath("e ~ f") == ("e/following-sibling::f")
         assert xpath("e ~ f:nth-child(3)") == (
             "e/following-sibling::f[count(preceding-sibling::*) = 2]"
         )
-        assert xpath("div#container p") == ("div[@id = 'container']/descendant-or-self::*/p")
+        assert xpath("div#container p") == (
+            "div[@id = 'container']/descendant-or-self::*/p"
+        )
         assert xpath("e:where(foo)") == "e[name() = 'foo']"
         assert xpath("e:where(foo, bar)") == "e[(name() = 'foo') or (name() = 'bar')]"
 
@@ -513,19 +564,32 @@ class TestCssselect(unittest.TestCase):
         assert xpath(r"[h\a0 ref]") == ("*[attribute::*[name() = 'h ref']]")  # h\xa0ref
         assert xpath(r"[h\]ref]") == ("*[attribute::*[name() = 'h]ref']]")
 
-        self.assertRaises(ExpressionError, xpath, ":fİrst-child")
-        self.assertRaises(ExpressionError, xpath, ":first-of-type")
-        self.assertRaises(ExpressionError, xpath, ":only-of-type")
-        self.assertRaises(ExpressionError, xpath, ":last-of-type")
-        self.assertRaises(ExpressionError, xpath, ":nth-of-type(1)")
-        self.assertRaises(ExpressionError, xpath, ":nth-last-of-type(1)")
-        self.assertRaises(ExpressionError, xpath, ":nth-child(n-)")
-        self.assertRaises(ExpressionError, xpath, ":after")
-        self.assertRaises(ExpressionError, xpath, ":lorem-ipsum")
-        self.assertRaises(ExpressionError, xpath, ":lorem(ipsum)")
-        self.assertRaises(ExpressionError, xpath, "::lorem-ipsum")
-        self.assertRaises(TypeError, GenericTranslator().css_to_xpath, 4)
-        self.assertRaises(TypeError, GenericTranslator().selector_to_xpath, "foo")
+        with pytest.raises(ExpressionError):
+            xpath(":fİrst-child")
+        with pytest.raises(ExpressionError):
+            xpath(":first-of-type")
+        with pytest.raises(ExpressionError):
+            xpath(":only-of-type")
+        with pytest.raises(ExpressionError):
+            xpath(":last-of-type")
+        with pytest.raises(ExpressionError):
+            xpath(":nth-of-type(1)")
+        with pytest.raises(ExpressionError):
+            xpath(":nth-last-of-type(1)")
+        with pytest.raises(ExpressionError):
+            xpath(":nth-child(n-)")
+        with pytest.raises(ExpressionError):
+            xpath(":after")
+        with pytest.raises(ExpressionError):
+            xpath(":lorem-ipsum")
+        with pytest.raises(ExpressionError):
+            xpath(":lorem(ipsum)")
+        with pytest.raises(ExpressionError):
+            xpath("::lorem-ipsum")
+        with pytest.raises(TypeError):
+            GenericTranslator().css_to_xpath(4)  # type: ignore[arg-type]
+        with pytest.raises(TypeError):
+            GenericTranslator().selector_to_xpath("foo")  # type: ignore[arg-type]
 
     def test_unicode(self) -> None:
         css = ".a\xc1b"
@@ -539,10 +603,18 @@ class TestCssselect(unittest.TestCase):
 
     def test_quoting(self) -> None:
         css_to_xpath = GenericTranslator().css_to_xpath
-        assert css_to_xpath('*[aval="\'"]') == ("""descendant-or-self::*[@aval = "'"]""")
-        assert css_to_xpath("*[aval=\"'''\"]") == ("""descendant-or-self::*[@aval = "'''"]""")
-        assert css_to_xpath("*[aval='\"']") == ("""descendant-or-self::*[@aval = '"']""")
-        assert css_to_xpath('*[aval=\'"""\']') == ('''descendant-or-self::*[@aval = '"""']''')
+        assert css_to_xpath('*[aval="\'"]') == (
+            """descendant-or-self::*[@aval = "'"]"""
+        )
+        assert css_to_xpath("*[aval=\"'''\"]") == (
+            """descendant-or-self::*[@aval = "'''"]"""
+        )
+        assert css_to_xpath("*[aval='\"']") == (
+            """descendant-or-self::*[@aval = '"']"""
+        )
+        assert css_to_xpath('*[aval=\'"""\']') == (
+            '''descendant-or-self::*[@aval = '"""']'''
+        )
         assert css_to_xpath(':scope > div[dataimg="<testmessage>"]') == (
             "descendant-or-self::*[1]/div[@dataimg = '<testmessage>']"
         )
@@ -569,33 +641,35 @@ class TestCssselect(unittest.TestCase):
                 self, xpath: XPathExpr, pseudo_element: PseudoElement
             ) -> XPathExpr:
                 if isinstance(pseudo_element, FunctionalPseudoElement):
-                    method_name = "xpath_%s_functional_pseudo_element" % (
+                    method_name = "xpath_{}_functional_pseudo_element".format(
                         pseudo_element.name.replace("-", "_")
                     )
                     method = getattr(self, method_name, None)
                     if not method:
                         raise ExpressionError(
-                            "The functional pseudo-element ::%s() is unknown" % pseudo_element.name
+                            f"The functional pseudo-element ::{pseudo_element.name}() is unknown"
                         )
                     xpath = method(xpath, pseudo_element.arguments)
                 else:
-                    method_name = "xpath_%s_simple_pseudo_element" % (
+                    method_name = "xpath_{}_simple_pseudo_element".format(
                         pseudo_element.replace("-", "_")
                     )
                     method = getattr(self, method_name, None)
                     if not method:
                         raise ExpressionError(
-                            "The pseudo-element ::%s is unknown" % pseudo_element
+                            f"The pseudo-element ::{pseudo_element} is unknown"
                         )
                     xpath = method(xpath)
                 return xpath
 
             # functional pseudo-class:
             # elements that have a certain number of attributes
-            def xpath_nb_attr_function(self, xpath: XPathExpr, function: Function) -> XPathExpr:
+            def xpath_nb_attr_function(
+                self, xpath: XPathExpr, function: Function
+            ) -> XPathExpr:
                 assert function.arguments[0].value
                 nb_attributes = int(function.arguments[0].value)
-                return xpath.add_condition("count(@*)=%d" % nb_attributes)
+                return xpath.add_condition(f"count(@*)={nb_attributes}")
 
             # pseudo-class:
             # elements that have 5 attributes
@@ -609,14 +683,16 @@ class TestCssselect(unittest.TestCase):
             ) -> XPathExpr:
                 attribute_name = arguments[0].value
                 other = XPathExpr(
-                    "@%s" % attribute_name,
+                    f"@{attribute_name}",
                     "",
                 )
                 return xpath.join("/", other)
 
             # pseudo-element:
             # element's text() nodes
-            def xpath_text_node_simple_pseudo_element(self, xpath: XPathExpr) -> XPathExpr:
+            def xpath_text_node_simple_pseudo_element(
+                self, xpath: XPathExpr
+            ) -> XPathExpr:
                 other = XPathExpr(
                     "text()",
                     "",
@@ -625,7 +701,9 @@ class TestCssselect(unittest.TestCase):
 
             # pseudo-element:
             # element's href attribute
-            def xpath_attr_href_simple_pseudo_element(self, xpath: XPathExpr) -> XPathExpr:
+            def xpath_attr_href_simple_pseudo_element(
+                self, xpath: XPathExpr
+            ) -> XPathExpr:
                 other = XPathExpr(
                     "@href",
                     "",
@@ -650,17 +728,17 @@ class TestCssselect(unittest.TestCase):
         )
         assert xpath(":scope") == "descendant-or-self::*[1]"
         assert xpath(":first-or-second[href]") == (
-            "descendant-or-self::*[(@id = 'first' or @id = 'second') " "and (@href)]"
+            "descendant-or-self::*[(@id = 'first' or @id = 'second') and (@href)]"
         )
 
         assert str(XPathExpr("", "", condition="@href")) == "[@href]"
 
         document = etree.fromstring(OPERATOR_PRECEDENCE_IDS)
-        sort_key = dict((el, count) for count, el in enumerate(document.iter())).__getitem__
+        sort_key = {el: count for count, el in enumerate(document.iter())}.__getitem__
 
-        def operator_id(selector: str) -> List[str]:
+        def operator_id(selector: str) -> list[str]:
             xpath = CustomTranslator().css_to_xpath(selector)
-            items = typing.cast(List["etree._Element"], document.xpath(xpath))
+            items = typing.cast("list[etree._Element]", document.xpath(xpath))
             items.sort(key=sort_key)
             return [element.get("id", "nil") for element in items]
 
@@ -669,9 +747,11 @@ class TestCssselect(unittest.TestCase):
         assert operator_id("[href]:first-or-second") == ["second"]
 
     def test_series(self) -> None:
-        def series(css: str) -> Optional[Tuple[int, int]]:
-            (selector,) = parse(":nth-child(%s)" % css)
-            args = typing.cast(FunctionalPseudoElement, selector.parsed_tree).arguments
+        def series(css: str) -> tuple[int, int] | None:
+            (selector,) = parse(f":nth-child({css})")
+            args = typing.cast(
+                "FunctionalPseudoElement", selector.parsed_tree
+            ).arguments
             try:
                 return parse_series(args)
             except ValueError:
@@ -698,12 +778,12 @@ class TestCssselect(unittest.TestCase):
 
     def test_lang(self) -> None:
         document = etree.fromstring(XMLLANG_IDS)
-        sort_key = dict((el, count) for count, el in enumerate(document.iter())).__getitem__
+        sort_key = {el: count for count, el in enumerate(document.iter())}.__getitem__
         css_to_xpath = GenericTranslator().css_to_xpath
 
-        def langid(selector: str) -> List[str]:
+        def langid(selector: str) -> list[str]:
             xpath = css_to_xpath(selector)
-            items = typing.cast(List["etree._Element"], document.xpath(xpath))
+            items = typing.cast("list[etree._Element]", document.xpath(xpath))
             items.sort(key=sort_key)
             return [element.get("id", "nil") for element in items]
 
@@ -714,28 +794,34 @@ class TestCssselect(unittest.TestCase):
         assert langid(":lang(ru)") == ["sixth"]
         assert langid(":lang('ZH')") == ["eighth"]
         assert langid(":lang(de) :lang(zh)") == ["eighth"]
-        assert langid(":lang(en), :lang(zh)") == ["first", "second", "third", "fourth", "eighth"]
+        assert langid(":lang(en), :lang(zh)") == [
+            "first",
+            "second",
+            "third",
+            "fourth",
+            "eighth",
+        ]
         assert langid(":lang(es)") == []
 
     def test_argument_types(self) -> None:
         class CustomTranslator(GenericTranslator):
             def __init__(self) -> None:
-                self.argument_types: List[str] = []
+                self.argument_types: list[str] = []
 
             def xpath_pseudo_element(
                 self, xpath: XPathExpr, pseudo_element: PseudoElement
             ) -> XPathExpr:
                 self.argument_types += typing.cast(
-                    FunctionalPseudoElement, pseudo_element
+                    "FunctionalPseudoElement", pseudo_element
                 ).argument_types()
                 return xpath
 
-        def argument_types(css: str) -> List[str]:
+        def argument_types(css: str) -> list[str]:
             translator = CustomTranslator()
             translator.css_to_xpath(css)
             return translator.argument_types
 
-        mappings: List[Tuple[str, List[str]]] = [
+        mappings: list[tuple[str, list[str]]] = [
             ("", []),
             ("ident", ["IDENT"]),
             ('"string"', ["STRING"]),
@@ -747,21 +833,21 @@ class TestCssselect(unittest.TestCase):
 
     def test_select(self) -> None:
         document = etree.fromstring(HTML_IDS)
-        sort_key = dict((el, count) for count, el in enumerate(document.iter())).__getitem__
+        sort_key = {el: count for count, el in enumerate(document.iter())}.__getitem__
         css_to_xpath = GenericTranslator().css_to_xpath
         html_css_to_xpath = HTMLTranslator().css_to_xpath
 
-        def select_ids(selector: str, html_only: bool) -> List[str]:
+        def select_ids(selector: str, html_only: bool) -> list[str]:
             xpath = css_to_xpath(selector)
-            items = typing.cast(List["etree._Element"], document.xpath(xpath))
+            items = typing.cast("list[etree._Element]", document.xpath(xpath))
             if html_only:
                 assert items == []
                 xpath = html_css_to_xpath(selector)
-                items = typing.cast(List["etree._Element"], document.xpath(xpath))
+                items = typing.cast("list[etree._Element]", document.xpath(xpath))
             items.sort(key=sort_key)
             return [element.get("id", "nil") for element in items]
 
-        def pcss(main: str, *selectors: str, **kwargs: bool) -> List[str]:
+        def pcss(main: str, *selectors: str, **kwargs: bool) -> list[str]:
             html_only = kwargs.pop("html_only", False)
             result = select_ids(main, html_only)
             for selector in selectors:
@@ -769,7 +855,14 @@ class TestCssselect(unittest.TestCase):
             return result
 
         all_ids = pcss("*")
-        assert all_ids[:6] == ["html", "nil", "link-href", "link-nohref", "nil", "outer-div"]
+        assert all_ids[:6] == [
+            "html",
+            "nil",
+            "link-href",
+            "link-nohref",
+            "nil",
+            "outer-div",
+        ]
         assert all_ids[-1:] == ["foobar-span"]
         assert pcss("div") == ["outer-div", "li-div", "foobar-div"]
         assert pcss("DIV", html_only=True) == [
@@ -780,7 +873,9 @@ class TestCssselect(unittest.TestCase):
         assert pcss("div div") == ["li-div"]
         assert pcss("div, div div") == ["outer-div", "li-div", "foobar-div"]
         assert pcss("a[name]") == ["name-anchor"]
-        assert pcss("a[NAme]", html_only=True) == ["name-anchor"]  # case-insensitive in HTML:
+        assert pcss("a[NAme]", html_only=True) == [
+            "name-anchor"
+        ]  # case-insensitive in HTML:
         assert pcss("a[rel]") == ["tag-anchor", "nofollow-anchor"]
         assert pcss('a[rel="tag"]') == ["tag-anchor"]
         assert pcss('a[href*="localhost"]') == ["tag-anchor"]
@@ -798,7 +893,10 @@ class TestCssselect(unittest.TestCase):
         assert pcss('*[lang|="en"]', '[lang|="en-US"]') == []
         assert pcss('*[lang|="e"]') == []
         # ... :lang() is not.
-        assert pcss(':lang("EN")', "*:lang(en-US)", html_only=True) == ["second-li", "li-div"]
+        assert pcss(':lang("EN")', "*:lang(en-US)", html_only=True) == [
+            "second-li",
+            "li-div",
+        ]
         assert pcss(':lang("e")', html_only=True) == []
         assert pcss(":scope > div") == []
         assert pcss(":scope body") == ["nil"]
@@ -852,7 +950,11 @@ class TestCssselect(unittest.TestCase):
             "seventh-li",
         ]
         assert pcss("li:nth-last-child(2n+2)") == ["second-li", "fourth-li", "sixth-li"]
-        assert pcss("li:nth-last-child(3n+1)") == ["first-li", "fourth-li", "seventh-li"]
+        assert pcss("li:nth-last-child(3n+1)") == [
+            "first-li",
+            "fourth-li",
+            "seventh-li",
+        ]
         assert pcss("ol:first-of-type") == ["first-ol"]
         assert pcss("ol:nth-child(1)") == []
         assert pcss("ol:nth-of-type(2)") == ["second-ol"]
@@ -875,7 +977,8 @@ class TestCssselect(unittest.TestCase):
         assert pcss("span:only-child") == ["foobar-span"]
         assert pcss("li div:only-child") == ["li-div"]
         assert pcss("div *:only-child") == ["li-div", "foobar-span"]
-        self.assertRaises(ExpressionError, pcss, "p *:only-of-type")
+        with pytest.raises(ExpressionError):
+            pcss("p *:only-of-type")
         assert pcss("p:only-of-type") == ["paragraph"]
         assert pcss("a:empty", "a:EMpty") == ["name-anchor"]
         assert pcss("li:empty") == ["third-li", "fourth-li", "fifth-li", "sixth-li"]
@@ -901,7 +1004,10 @@ class TestCssselect(unittest.TestCase):
         assert pcss('*:contains("E")') == []  # case-sensitive
         assert pcss(".a", ".b", "*.a", "ol.a") == ["first-ol"]
         assert pcss(".c", "*.c") == ["first-ol", "third-li", "fourth-li"]
-        assert pcss("ol *.c", "ol li.c", "li ~ li.c", "ol > li.c") == ["third-li", "fourth-li"]
+        assert pcss("ol *.c", "ol li.c", "li ~ li.c", "ol > li.c") == [
+            "third-li",
+            "fourth-li",
+        ]
         assert pcss("#first-li", "li#first-li", "*#first-li") == ["first-li"]
         assert pcss("li div", "li > div", "div div") == ["li-div"]
         assert pcss("div > div") == []
@@ -972,14 +1078,14 @@ class TestCssselect(unittest.TestCase):
 
     def test_select_shakespeare(self) -> None:
         document = html.document_fromstring(HTML_SHAKESPEARE)
-        body = typing.cast(List["etree._Element"], document.xpath("//body"))[0]
+        body = typing.cast("list[etree._Element]", document.xpath("//body"))[0]
         css_to_xpath = GenericTranslator().css_to_xpath
 
         basestring_ = (str, bytes)
 
         def count(selector: str) -> int:
             xpath = css_to_xpath(selector)
-            results = typing.cast(List["etree._Element"], body.xpath(xpath))
+            results = typing.cast("list[etree._Element]", body.xpath(xpath))
             assert not isinstance(results, basestring_)
             found = set()
             for item in results:
@@ -1427,7 +1533,7 @@ HTML_SHAKESPEARE = """
 </div>
 </body>
 </html>
-"""  # noqa: W191,E101
+"""
 
 
 if __name__ == "__main__":
